package telecom.sudparis.eu.paas.core.server.xml.manifest;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.StringWriter;
import java.util.ArrayList;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.stream.StreamSource;


/**
 * @author nguyen_n
 * 
 */
public class ManifestManagement {

	private String manifestName = "";
	private String manifestDesc = "";

	private String appName = "";
	private String appDesc = "";
	private String appStartCommand = "";
	private String appVersionName = "";
	private String appVersionLabel = "";
	private String appDeplName = "";
	private String appDeplType = "";
	private String appDeplLocation = "";
	private String appDeplTenaLevel = "";
	private ArrayList<PaasApplicationVersionInstanceType> listInstances = new ArrayList<PaasApplicationVersionInstanceType>();

	private String envName = "";
	private String envTemplateName = "";
	private String envTemplateMemory = "";
	private String envDesc = "";
	private ArrayList<PaasEnvironmentNodeType> envNodes = new ArrayList<PaasEnvironmentNodeType>();

	private ArrayList<PaasEnvironmentLinkType> envLinks = new ArrayList<PaasEnvironmentLinkType>();
	private ArrayList<PaasEnvironmentVariableType> envVariables = new ArrayList<PaasEnvironmentVariableType>();

	public ManifestManagement() {
		/*
		 * envNodes = new ArrayList<PaasEnvironmentNodeType>();
		 * PaasEnvironmentNodeType envNode = new PaasEnvironmentNodeType();
		 * envNode.setContentType("container"); envNode.setProvider("CF");
		 * envNode.setVersion("6.0"); envNode.setName("tomcat");
		 * envNodes.add(envNode);
		 * 
		 * envNode = new PaasEnvironmentNodeType();
		 * envNode.setContentType("database"); envNode.setProvider("OS");
		 * envNode.setVersion(""); envNode.setName("mysql");
		 * envNodes.add(envNode);
		 * 
		 * envVariables = new ArrayList<PaasEnvironmentVariableType>();
		 * PaasEnvironmentVariableType variable = new
		 * PaasEnvironmentVariableType(); variable.setName("");
		 * variable.setValue(""); envVariables.add(variable);
		 * 
		 * variable = new PaasEnvironmentVariableType(); variable.setName("");
		 * variable.setValue(""); envVariables.add(variable);
		 * 
		 * envLinks = new ArrayList<PaasEnvironmentLinkType>();
		 * PaasEnvironmentLinkType link = new PaasEnvironmentLinkType();
		 * link.setName("dbBinding"); link.setScript(""); envLinks.add(link);
		 * 
		 * link = new PaasEnvironmentLinkType(); link.setName("");
		 * link.setScript(""); envLinks.add(link);
		 * 
		 * listInstances = new ArrayList<PaasApplicationVersionInstanceType>();
		 * PaasApplicationVersionInstanceType instance = new
		 * PaasApplicationVersionInstanceType();
		 * instance.setDefaultInstance(true); instance.setInitialState(1);
		 * instance.setName("Instance1"); instance.setDisk(2);
		 * instance.setMemory(256); listInstances.add(instance);
		 * 
		 * instance = new PaasApplicationVersionInstanceType();
		 * instance.setDefaultInstance(true); instance.setInitialState(1);
		 * instance.setName("Instance2"); instance.setDisk(2);
		 * instance.setMemory(128); listInstances.add(instance);
		 */

	}

	public ManifestManagement(String manifestName, String manifestDesc,
			String appName, String appDesc, String appStartCommand,
			String appVersionName, String appVersionLabel, String appDeplName,
			String appDeplType, String appDeplLocation,
			String appDeplTenaLevel,
			ArrayList<PaasApplicationVersionInstanceType> listInstances,
			String envName, String envTemplateName, String envTemplateMemory,
			String envDesc, ArrayList<PaasEnvironmentNodeType> envNodes,
			ArrayList<PaasEnvironmentLinkType> envLinks,
			ArrayList<PaasEnvironmentVariableType> envVariables
	) {
		this.manifestName = manifestName;
		this.manifestDesc = manifestDesc;
		this.appName = appName;
		this.appDesc = appDesc;
		this.appStartCommand = appStartCommand;
		this.appVersionName = appVersionName;
		this.appVersionLabel = appVersionLabel;
		this.appDeplName = appDeplName;
		this.appDeplType = appDeplType;
		this.appDeplLocation = appDeplLocation;
		this.appDeplTenaLevel = appDeplTenaLevel;
		this.listInstances = listInstances;
		this.envName = envName;
		this.envTemplateName = envTemplateName;
		this.envTemplateMemory = envTemplateMemory;
		this.envDesc = envDesc;
		this.envNodes = envNodes;
		this.envLinks = envLinks;
		this.envVariables = envVariables;
	}

	public String getManifest() {
		// create JAXB context and instantiate marshaller
		JAXBContext context;

		// System.out.println(fileToStoreDB);
		try {
			context = JAXBContext
					.newInstance("telecom.sudparis.eu.paas.core.server.xml.manifest");

			Marshaller m = context.createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);

			PaasApplicationManifestType manifest = new PaasApplicationManifestType();

			manifest.setName(manifestName);
			manifest.setDescription(manifestDesc);

			// create environment
			PaasEnvironmentType eType = new PaasEnvironmentType();

			eType.setName(envName);
			eType.setTemplate(envTemplateName);

			PaasEnvironmentTemplateType eTemp = new PaasEnvironmentTemplateType();
			eTemp.setName(envTemplateName);
			eTemp.setMemory(envTemplateMemory.trim().equals("") ? 0 : Integer
					.parseInt(envTemplateMemory));
			eTemp.setDescription(envDesc);
			eTemp.setPaasEnvironmentNode(envNodes);

			PaasEnvironmentRelationType envRelation = new PaasEnvironmentRelationType();
			envRelation.setPaasEnvironmentLinks(envLinks);
			eTemp.setPassEnvironmentRelation(envRelation);

			PaasEnvironmentConfigurationType envConfig = new PaasEnvironmentConfigurationType();
			envConfig.setPaasEnvironmentVariables(envVariables);
			eTemp.setPassEnvironmentConfig(envConfig);

			eType.setPaasEnvironmentTemplate(eTemp);
			manifest.setPaasEnvironment(eType);

			// create application
			PaasApplicationType aType = new PaasApplicationType();

			aType.setDescription(appDesc);
			aType.setName(appName);
			aType.setEnvironment(envName);
			aType.setStartCommand(appStartCommand);

			PaasApplicationVersionType aVersion = new PaasApplicationVersionType();
			aVersion.setLabel(appVersionLabel);
			aVersion.setName(appVersionName);

			PaasApplicationDeployableType aDeploy = new PaasApplicationDeployableType();
			aDeploy.setContentType(appDeplType);
			aDeploy.setLocation(appDeplLocation);
			aDeploy.setMultitenancyLevel(appDeplTenaLevel);
			aDeploy.setName(appDeplName);

			aVersion.setPaasApplicationDeployable(aDeploy);
			aVersion.setPaasApplicationVersionInstance(listInstances);
			aType.setPaasApplicationVersion(aVersion);

			manifest.setPaasApplication(aType);

			// generate XML manifest string
			StringWriter sw = new StringWriter();
			// Write to System.out
			m.marshal(manifest, sw);

			return sw.toString();

		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	public String getApplicationOnly() {
		// create JAXB context and instantiate marshaller
		JAXBContext context;

		// System.out.println(fileToStoreDB);
		try {
			context = JAXBContext
					.newInstance("telecom.sudparis.eu.paas.core.server.xml.manifest");

			Marshaller m = context.createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);

			PaasApplicationManifestType manifest = new PaasApplicationManifestType();

			manifest.setName(manifestName);
			manifest.setDescription(manifestDesc);

			// create application
			PaasApplicationType aType = new PaasApplicationType();

			aType.setDescription(appDesc);
			aType.setName(appName);
			aType.setEnvironment(envName);
			aType.setStartCommand(appStartCommand);

			PaasApplicationVersionType aVersion = new PaasApplicationVersionType();
			aVersion.setLabel(appVersionLabel);
			aVersion.setName(appVersionName);

			PaasApplicationDeployableType aDeploy = new PaasApplicationDeployableType();
			aDeploy.setContentType(appDeplType);
			aDeploy.setLocation(appDeplLocation);
			aDeploy.setMultitenancyLevel(appDeplTenaLevel);
			aDeploy.setName(appDeplName);

			aVersion.setPaasApplicationDeployable(aDeploy);
			aVersion.setPaasApplicationVersionInstance(listInstances);
			aType.setPaasApplicationVersion(aVersion);

			manifest.setPaasApplication(aType);

			// generate XML manifest string
			StringWriter sw = new StringWriter();
			// Write to System.out
			m.marshal(manifest, sw);

			return sw.toString();

		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	public String getEnvironmentOnly() {
		// create JAXB context and instantiate marshaller
		JAXBContext context;

		// System.out.println(fileToStoreDB);
		try {
			context = JAXBContext
					.newInstance("telecom.sudparis.eu.paas.core.server.xml.manifest");

			Marshaller m = context.createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);

			PaasApplicationManifestType manifest = new PaasApplicationManifestType();

			manifest.setName(manifestName);
			manifest.setDescription(manifestDesc);

			// create environment
			PaasEnvironmentType eType = new PaasEnvironmentType();

			eType.setName(envName);
			eType.setTemplate(envTemplateName);

			PaasEnvironmentTemplateType eTemp = new PaasEnvironmentTemplateType();
			eTemp.setName(envTemplateName);
			eTemp.setMemory(envTemplateMemory.trim().equals("") ? 0 : Integer
					.parseInt(envTemplateMemory));
			eTemp.setDescription(envDesc);
			eTemp.setPaasEnvironmentNode(envNodes);

			PaasEnvironmentRelationType envRelation = new PaasEnvironmentRelationType();
			envRelation.setPaasEnvironmentLinks(envLinks);
			eTemp.setPassEnvironmentRelation(envRelation);

			PaasEnvironmentConfigurationType envConfig = new PaasEnvironmentConfigurationType();
			envConfig.setPaasEnvironmentVariables(envVariables);
			eTemp.setPassEnvironmentConfig(envConfig);

			eType.setPaasEnvironmentTemplate(eTemp);
			manifest.setPaasEnvironment(eType);

			// generate XML manifest string
			StringWriter sw = new StringWriter();
			// Write to System.out
			m.marshal(manifest, sw);

			return sw.toString();

		} catch (JAXBException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	public void parseManifest(String manifestDescription) {
		try {
			if (manifestDescription != null) {
				InputStream is = new ByteArrayInputStream(
						manifestDescription.getBytes());

				JAXBContext jaxbContext;
				PaasApplicationManifestType manifest = new PaasApplicationManifestType();
				try {
					jaxbContext = JAXBContext
							.newInstance("telecom.sudparis.eu.paas.core.server.xml.manifest");
					Unmarshaller jaxbUnmarshaller = jaxbContext
							.createUnmarshaller();

					JAXBElement<PaasApplicationManifestType> root = jaxbUnmarshaller
							.unmarshal(new StreamSource(is),
									PaasApplicationManifestType.class);

					manifest = root.getValue();

				} catch (JAXBException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

				manifestName = manifest.getName();
				manifestDesc = manifest.getDescription();

				if (manifest.getPaasApplication() != null) {
					appName = manifest.getPaasApplication().getName();
					appDesc = manifest.getPaasApplication().getDescription();
					appStartCommand = manifest.getPaasApplication()
							.getStartCommand();
					if (manifest.getPaasApplication()
							.getPaasApplicationVersion() != null) {
						appVersionName = manifest.getPaasApplication()
								.getPaasApplicationVersion().getName();
						if (manifest.getPaasApplication()
								.getPaasApplicationVersion().getLabel() != null)
							appVersionLabel = manifest.getPaasApplication()
									.getPaasApplicationVersion().getLabel()
									.toString();
						if (manifest.getPaasApplication()
								.getPaasApplicationVersion()
								.getPaasApplicationDeployable() != null) {
							appDeplName = manifest.getPaasApplication()
									.getPaasApplicationVersion()
									.getPaasApplicationDeployable().getName();
							appDeplType = manifest.getPaasApplication()
									.getPaasApplicationVersion()
									.getPaasApplicationDeployable()
									.getContentType();
							appDeplLocation = manifest.getPaasApplication()
									.getPaasApplicationVersion()
									.getPaasApplicationDeployable()
									.getLocation();
							appDeplTenaLevel = manifest.getPaasApplication()
									.getPaasApplicationVersion()
									.getPaasApplicationDeployable()
									.getMultitenancyLevel();
						}
						listInstances = manifest.getPaasApplication()
								.getPaasApplicationVersion()
								.getPaasApplicationVersionInstance();
					}
				}

				if (manifest.getPaasEnvironment() != null) {
					envName = manifest.getPaasEnvironment().getName();
					envTemplateName = manifest.getPaasEnvironment()
							.getTemplate();
					if (manifest.getPaasEnvironment()
							.getPaasEnvironmentTemplate() != null) {
						if (manifest.getPaasEnvironment()
								.getPaasEnvironmentTemplate().getMemory() != null)
							envTemplateMemory = manifest.getPaasEnvironment()
									.getPaasEnvironmentTemplate().getMemory()
									.toString();

						envDesc = manifest.getPaasEnvironment()
								.getPaasEnvironmentTemplate().getDescription();
						envNodes = manifest.getPaasEnvironment()
								.getPaasEnvironmentTemplate()
								.getPaasEnvironmentNode();

						if (manifest.getPaasEnvironment()
								.getPaasEnvironmentTemplate()
								.getPassEnvironmentRelation() != null)
							envLinks = manifest.getPaasEnvironment()
									.getPaasEnvironmentTemplate()
									.getPassEnvironmentRelation()
									.getPaasEnvironmentLinks();
						if (manifest.getPaasEnvironment()
								.getPaasEnvironmentTemplate()
								.getPassEnvironmentConfig() != null)
							envVariables = manifest.getPaasEnvironment()
									.getPaasEnvironmentTemplate()
									.getPassEnvironmentConfig()
									.getPaasEnvironmentVariables();
					}
				}
			}
		} catch (Exception e) {
			System.out.println("Failed to create the environment: "
					+ e.getMessage());
			e.printStackTrace();
		}
	}

	public void parseApplicationOnly(String description) {
		try {
			if (description != null) {
				InputStream is = new ByteArrayInputStream(
						description.getBytes());

				JAXBContext jaxbContext;
				PaasApplicationManifestType manifest = new PaasApplicationManifestType();
				try {
					jaxbContext = JAXBContext
							.newInstance("telecom.sudparis.eu.paas.core.server.xml.manifest");
					Unmarshaller jaxbUnmarshaller = jaxbContext
							.createUnmarshaller();

					JAXBElement<PaasApplicationManifestType> root = jaxbUnmarshaller
							.unmarshal(new StreamSource(is),
									PaasApplicationManifestType.class);

					manifest = root.getValue();

				} catch (JAXBException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

				manifestName = manifest.getName();
				manifestDesc = manifest.getDescription();

				if (manifest.getPaasApplication() != null) {
					appName = manifest.getPaasApplication().getName();
					appDesc = manifest.getPaasApplication().getDescription();
					appStartCommand = manifest.getPaasApplication()
							.getStartCommand();
					if (manifest.getPaasApplication()
							.getPaasApplicationVersion() != null) {
						appVersionName = manifest.getPaasApplication()
								.getPaasApplicationVersion().getName();
						if (manifest.getPaasApplication()
								.getPaasApplicationVersion().getLabel() != null)
							appVersionLabel = manifest.getPaasApplication()
									.getPaasApplicationVersion().getLabel()
									.toString();
						if (manifest.getPaasApplication()
								.getPaasApplicationVersion()
								.getPaasApplicationDeployable() != null) {
							appDeplName = manifest.getPaasApplication()
									.getPaasApplicationVersion()
									.getPaasApplicationDeployable().getName();
							appDeplType = manifest.getPaasApplication()
									.getPaasApplicationVersion()
									.getPaasApplicationDeployable()
									.getContentType();
							appDeplLocation = manifest.getPaasApplication()
									.getPaasApplicationVersion()
									.getPaasApplicationDeployable()
									.getLocation();
							appDeplTenaLevel = manifest.getPaasApplication()
									.getPaasApplicationVersion()
									.getPaasApplicationDeployable()
									.getMultitenancyLevel();
						}
						listInstances = manifest.getPaasApplication()
								.getPaasApplicationVersion()
								.getPaasApplicationVersionInstance();
					}
				}
			}
		} catch (Exception e) {
			System.out.println("Failed to create the environment: "
					+ e.getMessage());
			e.printStackTrace();
		}
	}

	public void parseEnvironmentOnly(String description) {
		try {
			if (description != null) {
				InputStream is = new ByteArrayInputStream(
						description.getBytes());

				JAXBContext jaxbContext;
				PaasApplicationManifestType manifest = new PaasApplicationManifestType();
				try {
					jaxbContext = JAXBContext
							.newInstance("telecom.sudparis.eu.paas.core.server.xml.manifest");
					Unmarshaller jaxbUnmarshaller = jaxbContext
							.createUnmarshaller();

					JAXBElement<PaasApplicationManifestType> root = jaxbUnmarshaller
							.unmarshal(new StreamSource(is),
									PaasApplicationManifestType.class);

					manifest = root.getValue();

				} catch (JAXBException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

				manifestName = manifest.getName();
				manifestDesc = manifest.getDescription();

				if (manifest.getPaasEnvironment() != null) {
					envName = manifest.getPaasEnvironment().getName();
					envTemplateName = manifest.getPaasEnvironment()
							.getTemplate();
					if (manifest.getPaasEnvironment()
							.getPaasEnvironmentTemplate() != null) {
						if (manifest.getPaasEnvironment()
								.getPaasEnvironmentTemplate().getMemory() != null)
							envTemplateMemory = manifest.getPaasEnvironment()
									.getPaasEnvironmentTemplate().getMemory()
									.toString();

						envDesc = manifest.getPaasEnvironment()
								.getPaasEnvironmentTemplate().getDescription();
						envNodes = manifest.getPaasEnvironment()
								.getPaasEnvironmentTemplate()
								.getPaasEnvironmentNode();

						if (manifest.getPaasEnvironment()
								.getPaasEnvironmentTemplate()
								.getPassEnvironmentRelation() != null)
							envLinks = manifest.getPaasEnvironment()
									.getPaasEnvironmentTemplate()
									.getPassEnvironmentRelation()
									.getPaasEnvironmentLinks();
						if (manifest.getPaasEnvironment()
								.getPaasEnvironmentTemplate()
								.getPassEnvironmentConfig() != null)
							envVariables = manifest.getPaasEnvironment()
									.getPaasEnvironmentTemplate()
									.getPassEnvironmentConfig()
									.getPaasEnvironmentVariables();
					}
				}
			}
		} catch (Exception e) {
			System.out.println("Failed to create the environment: "
					+ e.getMessage());
			e.printStackTrace();
		}
	}

	public String getManifestName() {
		return manifestName;
	}

	public void setManifestName(String manifestName) {
		this.manifestName = manifestName;
	}

	public String getManifestDesc() {
		return manifestDesc;
	}

	public void setManifestDesc(String manifestDesc) {
		this.manifestDesc = manifestDesc;
	}

	public String getAppName() {
		return appName;
	}

	public void setAppName(String appName) {
		this.appName = appName;
	}

	public String getAppDesc() {
		return appDesc;
	}

	public void setAppDesc(String appDesc) {
		this.appDesc = appDesc;
	}

	public String getAppStartCommand() {
		return appStartCommand;
	}

	public void setAppStartCommand(String appStartCommand) {
		this.appStartCommand = appStartCommand;
	}

	public String getAppVersionName() {
		return appVersionName;
	}

	public void setAppVersionName(String appVersionName) {
		this.appVersionName = appVersionName;
	}

	public String getAppVersionLabel() {
		return appVersionLabel;
	}

	public void setAppVersionLabel(String appVersionLabel) {
		this.appVersionLabel = appVersionLabel;
	}

	public String getAppDeplName() {
		return appDeplName;
	}

	public void setAppDeplName(String appDeplName) {
		this.appDeplName = appDeplName;
	}

	public String getAppDeplType() {
		return appDeplType;
	}

	public void setAppDeplType(String appDeplType) {
		this.appDeplType = appDeplType;
	}

	public String getAppDeplLocation() {
		return appDeplLocation;
	}

	public void setAppDeplLocation(String appDeplLocation) {
		this.appDeplLocation = appDeplLocation;
	}

	public String getAppDeplTenaLevel() {
		return appDeplTenaLevel;
	}

	public void setAppDeplTenaLevel(String appDeplTenaLevel) {
		this.appDeplTenaLevel = appDeplTenaLevel;
	}

	public ArrayList<PaasApplicationVersionInstanceType> getListInstances() {
		return listInstances;
	}

	public void setListInstances(
			ArrayList<PaasApplicationVersionInstanceType> listInstances) {
		this.listInstances = listInstances;
	}

	public String getEnvName() {
		return envName;
	}

	public void setEnvName(String envName) {
		this.envName = envName;
	}

	public String getEnvTemplateName() {
		return envTemplateName;
	}

	public void setEnvTemplateName(String envTemplateName) {
		this.envTemplateName = envTemplateName;
	}

	public String getEnvTemplateMemory() {
		return envTemplateMemory;
	}

	public void setEnvTemplateMemory(String envTemplateMemory) {
		this.envTemplateMemory = envTemplateMemory;
	}

	public String getEnvDesc() {
		return envDesc;
	}

	public void setEnvDesc(String envDesc) {
		this.envDesc = envDesc;
	}

	public ArrayList<PaasEnvironmentNodeType> getEnvNodes() {
		return envNodes;
	}

	public void setEnvNodes(ArrayList<PaasEnvironmentNodeType> envNodes) {
		this.envNodes = envNodes;
	}

	public ArrayList<PaasEnvironmentLinkType> getEnvLinks() {
		return envLinks;
	}

	public void setEnvLinks(ArrayList<PaasEnvironmentLinkType> envLinks) {
		this.envLinks = envLinks;
	}

	public ArrayList<PaasEnvironmentVariableType> getEnvVariables() {
		return envVariables;
	}

	public void setEnvVariables(
			ArrayList<PaasEnvironmentVariableType> envVariables) {
		this.envVariables = envVariables;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		ManifestManagement mm = new ManifestManagement();

		String s = "<?xml version=\"1.0\" encoding=\"UTF8\"?>"
				+ "<paas_application_manifest name=\"ServletSampleApplicationManifest\">"
				+ "<description>This manifest describes a Sample Servlet.</description>"
				+ "	<paas_application name=\"ServletSampleApplication\" environement=\"JavaWebEnv\">"
				+ "		<description>Sample Servlet application description.</description>"
				+ "		<paas_application_version name=\"version1.0\" label=\"1.0\">"
				+ "			<paas_application_deployable name=\"SampleServlet.war\" content_type=\"artifact\" location=\"[location]\" multitenancy_level=\"SharedInstance\"/>"
				+ "			<paas_application_version_instance name=\"Instance1\" initial_state=\"1\" default_instance=\"true\"/>"
				+ "		</paas_application_version>"
				+ "	</paas_application>"
				+ "	<paas_environment name=\"JavaWebEnv\" template=\"TomcatEnvTemp\">"
				+ "		<paas_environment_template name=\"TomcatEnvTemp\" memory=\"65\">"
				+ "  		  <description>TomcatServerEnvironmentTemplate</description>"
				+ "		  <paas_environment_node content_type=\"container\" name=\"tomcat\" version=\"\" provider=\"CF\"/>"
				+ "		</paas_environment_template>" + "	</paas_environment>"
				+ "</paas_application_manifest>";

		//System.out.print(mm.getApplicationOnly());
		mm.parseManifest(s);
	}

}
